## Charm for IBM MQ

### Overview

IBM MQ

IBM MQ provides messaging services to transport multiple types of data.
MQ Runtime, Server and Sample components from IBM MQ product can be deployed using this charm.


This charm installs IBM MQ product and configures it's IP Address and default
port number (1414). MQ Administrators can setup queue managers
and queues. This will allow IBM MQ client application to transmit/receive messages.

More information on IBM MQ available at the [Product Page](http://www-03.ibm.com/software/products/en/ibm-mq) and [IBM Knowledge Center](http://www.ibm.com/support/knowledgecenter/SSFKSJ_8.0.0/com.ibm.mq.helphome.v80.doc/WelcomePagev8r0.htm)

### Usage

Download your licensed IBM MQ software and host it in a way that can be downloaded using curl.
curl-url and curl-opts are two main options we are using from ibm-base-layer to download the ibm product as IBM-MQ is built on top of the ibm-base layer [More Info](http://interfaces.juju.solutions/layer/ibm-base/)

### Deploy

Run the following commands to deploy this charm:
           
    juju deploy ibm-mq

Here the charm will wait until the user accepts the license. If you agree to the license, run the following command.

    juju set-config ibm-mq license_accepted="True"

IBM MQ charm also needs an host from where the packages can be downloaded (the user has to keep the MQ packages on this host server), from the directory where the package is kept, user name and password to connect to the host server (as mentioned in the creating repositories section).

We have to provide values for curl-url and curl-opt to download the product.
curl-url with host name,package_dir, package name, checksum value, curl-opts with user name and password, to proceed with the installation. To provide these run the following command:

    juju set-config ibm-mq curl_url="<hostname>/<package_dir><package_name>?<sha512=value>"
    juju set-config ibm-mq curl_opts="-u <username>:<password>"


For eg:
    juju set-config ibm-mq curl_url="http://1.2.3.4/debs/MQ/WS_MQ_V8.0.0.2_LINUX_ON_X86_64_I.tar.gz?sha512=cd604a577ae51d93fc384241ce2faf21927b202d0a9e5f9b9a25caf4460fd6c34f32649caef8ecd543c244413788fe105987d16553e2d4754ef02d924843e95f"

    juju set-config ibm-mq curl_opts="-u root:root123"

Setting the license-accepted to False will uninstall the product:

     juju set-config ibm-mq license_accepted="False"
	  
	  
### Update the following system configuration manually:

In order to create MQ objects like queue, it is required that some
	of the system configurations be updated. Details provided at
	[IBM Knowledge Center][mq-config-info].

  * In `/etc/sysctl.conf`:

        fs.file-max = 524288
        kernel.shmmax = 268435456

### Relation with other services:

 ibm-mq can have relation with other services to get the service specific queue manager. Per example if ibm-mq gets relation with ibm-was then, ibm-was specific queue manger will be created like 'ibm-wasQM'.
 
    juju add-relation ibm-mq:messaging ibm-was-base:wasmessaging
 or     
 
    juju add-relation ibm-mq ibm-was-base
	
IBM-MQ sets Queue Manager name, Queue Name, IP Address and Port for the connected services.
	It creates a Queue Manager for the consumer application and a local queue i.e QUEUE1 for above Queue Manager.
	But as a Queue Name it sets a filename where MQSC Commands can be edited for further operation.
      
on removal of relation with the service with which already relation was established, the service specific queue manger will be removed i.e in the above case 'ibm-wasQM' will be removed.

     juju remove-relation ibm-mq:messaging ibm-was-base:wasmessaging
or  
    
     juju remove-relation ibm-mq ibm-was-base
##### The Charm updates the following system configuration:

In order to create MQ objects like queue, it is required that some
of the system configurations be updated. Details provided at
[IBM Knowledge Center][mq-config-info].

  * Adds user `ubuntu` to group `mqm`.

  * In `/etc/security/limits.conf` if hard nofile and soft nofile limit for user `mqm` are  less than 10240:

        mqm              hard    nofile          10240
        mqm              soft    nofile          10240
  
  * Updates `/etc/pam.d/common-session` file to take the above configurations to take effect

Verify these changes have been made successfully if MQ throws up any errors.


#### Configuration

**license_accepted** (boolean) Some IBM charms require acceptance of a license before installation can proceed. If required, setting this option to True indicates that you have read and accepted the IBM terms and conditions found in the license file referenced by the charm. 

 **curl_url** 
 
 For downloading package we need to set curl_url with following options.
  - *hostname* - The web server host name from which IBM MQ installation packages can be downloaded.
  - *package_dir* - The package directory path in the web server.
  - *package name* - The IBM MQ Package name.
  - *checksum value* - Checksum value to check integrity of IBM MQ package. The Charm
    uses sha512sum to check the integrity. If empty, it does not carry out the
    integrity check. Use `sha512sum <Package Location/Package Name>`
    to find out Checksum value for downloaded package. 

**curl_opts**

   We need to set curl_opts with following options. - username - User name of the webserver/sftp host. - password - Password of the webserver/sftp host.

##### Known Limitations:

Local providers does not allow updates to system configuration using sysctl
command. These configurations will need be done manually for MQ to function.
The required configurations are provided in this document.

### IBM MQ Information

(1) General Information
Details about MQ available [here](http://www-01.ibm.com/software/integration/wmq/library/).
Details about MQ Version 8.0 available at [IBM Knowledge Center](http://www.ibm.com/support/knowledgecenter/SSFKSJ_8.0.0/com.ibm.mq.helphome.v80.doc/WelcomePagev8r0.htm).

(2) Download Information
Information on procuring MQ product is available at the [Product Page](http://www-03.ibm.com/software/products/en/ibm-mq) and at the [Passport Advantage Site](http://www-01.ibm.com/software/passportadvantage/).

A development version for x86_64 is available [here](http://www14.software.ibm.com/cgi-bin/weblap/lap.pl?popup=Y&li_formnum=L-APIG-9BUHAE&accepted_url=http://public.dhe.ibm.com/ibmdl/export/pub/software/websphere/messaging/mqadv/mqadv_dev80_linux_x86-64.tar.gz). After clicking on the above link and if you agree to the license, click on 'I agree' link to download the IBM MQ package.
A 90 days trial version is available [here](http://www-03.ibm.com/software/products/en/ibm-mq).

(3) License Information
License information for IBM MQ is available as part of the downloaded product package. The license can be viewed by running the following command after extracting IBM MQ.

./mqlicense.sh -text_only

License for the development version is available [here](http://www14.software.ibm.com/cgi-bin/weblap/lap.pl?popup=Y&li_formnum=L-APIG-9BUHAE&accepted_url=http://public.dhe.ibm.com/ibmdl/export/pub/software/websphere/messaging/mqadv/mqadv_dev80_linux_x86-64.tar.gz).

Contact Information
For issues with this charm, please contact jujusupp@us.ibm.com
