#!/bin/bash

set -ex
source charms.reactive.sh

ARCHIVE_DIR=$CHARM_DIR/files/archives
MQ_INSTALL_PATH=/opt/mqm
ARCHITECTURE=`uname -m`
RPM_INSTALL_ARG="-ivh --nodeps --force-debian"
relatedService=""
QMA=""
QM_Name=""
hostname=`unit-get private-address`
	if [ "$ARCHITECTURE" = "ppc64le" ]; then
	RPM_INSTALL_ARG="-ivh --nodeps --force-debian --ignorearch"
	fi


# Check whether IBM MQ is installed

is_mq_installed()
{
	if [ -d $MQ_INSTALL_PATH/bin ]; then
	source $MQ_INSTALL_PATH/bin/setmqenv -s
		if [ $? == 0 ]; then
		source $MQ_INSTALL_PATH/bin/setmqenv -s
		echo "True"
		fi
	else
	echo "False"
	fi
}

# Remove MQ, if installed
remove_software()
{

	mq_installed=`is_mq_installed`
	if  [ $mq_installed == True ]; then
		juju-log "Removing IBM MQ software."
		# Stop all queues
		questr="QMNAME("
		for queue in `$MQ_INSTALL_PATH/bin/dspmq -o installation | cut -d' ' -f1 `; 
		do
			# Get the queue manager name 
			queue_mgr_name=${queue:${#questr}:${#queue}-${#questr}-1}
			juju-log "Stopping queue manager $queue_mgr_name"
			set +e
			su -l mqm -c "$MQ_INSTALL_PATH/bin/endmqm -w $queue_mgr_name"
			su -l mqm -c "$MQ_INSTALL_PATH/bin/endmqlsr -w -m $queue_mgr_name"
			set -e
		done
		# Get list of packages and remove them
		mq_rpms="`rpm -qa | grep MQSeries`"
		juju-log "Removing package(s) $mq_rpms"
		rpm -ev --force-debian $mq_rpms 
	fi

}

# Update system configuration after installing MQ
configure_system()
{
	juju-log "IBM MQ: Updating system configuration."
	# Some containers do not allow system updates.
	# Prevent the script from failing in such cases
	set +e

	# Add user ubuntu to mqm group
	adduser ubuntu mqm

	# Update mqm file limits

	hard_nf=`su - mqm -c "ulimit -Hn"`
	soft_nf=`su - mqm -c "ulimit -Sn"`

	if [ $hard_nf -lt 10240 -o $soft_nf -lt 10240 ]; then
		if [ "`grep mqm  /etc/security/limits.conf`"=="" ]; then
			sed -i 's/# End of file/mqm              hard    nofile          10240\nmqm              soft    nofile          10240\n# End of file/' /etc/security/limits.conf
		fi
		#Update /etc/pam.d/common-session file to take effect the above changes
		result_search=`grep 'session required pam_limits.so' /etc/pam.d/common-session`
		if [ -z "$result_search" ]; then
			sed -i 's/# end of pam-auth-update config/session required pam_limits.so\n# end of pam-auth-update config/' /etc/pam.d/common-session
		fi
	fi

	set -e
	juju-log "IBM MQ: Completed system configuration update."

}

# create Queue manager and Queue 
create_QM_Queue()
{

	juju-log "IBM MQ: Verifying installation."

	relatedService=$1
	consumer_hostname=$2

	QMA="${relatedService//-}.queue.manager"

	# Prevent the script from failing on failure. 
	# It could because configure_system call failed
	set +e

	# Run as mqm user as root will not work

	# Create queue manager
	juju-log "IBM MQ: Create queue manager $QMA."
	su -l mqm -c "$MQ_INSTALL_PATH/bin/crtmqm $QMA"
	rc=$?
	if [ $rc == 0 ]; then
		juju-log "IBM MQ: queue manager $QMA created."
		QM_create=1
	elif [ $rc == 8 ]; then
		QM_exists=1
	else
		juju-log "IBM MQ: Failed to create queue manager $QMA."
	fi

	# Start queue manager
	if [ $QM_create -eq 1 ]; then
		juju-log "IBM MQ: Starting queue manager $QMA."
		su -l mqm -c "$MQ_INSTALL_PATH/bin/strmqm $QMA"
		if [ $? == 0 ]; then
			juju-log "IBM MQ: Queue manager $QMA started."
		else
			juju-log "IBM MQ: Failed to start queue manager $QMA."
	
		fi
	
		# set the port to listen queue manager
		new_port=1414
		free_port=0
		is_free=0
	
		#Check whether the port number is already in use
		while [ $free_port -eq 0 ]
		do
	
			#is_free=`netstat -lnp | grep $new_port | cut -d":" -f2 | cut -d" " -f1`
			is_free=`netstat -an | grep $new_port | grep LISTEN`
			if [ -z "$is_free" ]; then
				free_port=1
			else
				new_port=$((new_port+1))
			fi
	
		done

		juju-log "IBM MQ: Setting port to $new_port"
	
		#start queue manager in listener mode
		juju-log "IBM MQ: Starting queue manager $QMA in listener mode"
		su -l mqm -c "$MQ_INSTALL_PATH/bin/runmqlsr -m $QMA -t tcp -p $new_port" &
		if [ $? == 0 ]; then
			juju-log "IBM MQ: Queue manager $QMA listening on $new_port"
		else
			juju-log "IBM MQ: Queue manager $QMA failed to listen"
		fi

		# Create queue and setup the queue configuration for consumer service 
		juju-log "IBM MQ: Creating queue."

		sudo bash -c "cat > $CHARM_DIR/files/archives/${relatedService//-}_mq_create_queue.mqsc << EOF
DEFINE QLOCAL (QUEUE1)
DEFINE CHANNEL ('MDB.SVRCONN') CHLTYPE(SVRCONN) TRPTYPE(TCP)
SET CHLAUTH ('MDB.SVRCONN') TYPE(ADDRESSMAP) ADDRESS($consumer_hostname) MCAUSER('mqm')
SET AUTHREC OBJTYPE(QMGR) PRINCIPAL('mqm') AUTHADD(CONNECT,INQ)
SET AUTHREC PROFILE('QUEUE1') OBJTYPE(QUEUE) PRINCIPAL('mqm') AUTHADD(PUT,GET,INQ,BROWSE)
SET AUTHREC PROFILE('SYSTEM.BASE.TOPIC') OBJTYPE(TOPIC) PRINCIPAL('mqm') AUTHADD(PUB, SUB)
EOF"
		if [ $? == 0 ]; then
			# Create file to delete the above queue when relation breaks
			sudo bash -c "echo 'DELETE QLOCAL(QUEUE1) PURGE' > $CHARM_DIR/files/archives/${relatedService//-}_mq_delete_queue.mqsc"
		fi

		if [ -f $CHARM_DIR/files/archives/${relatedService//-}_mq_create_queue.mqsc ]; then

			su -l mqm -c "$MQ_INSTALL_PATH/bin/runmqsc $QMA < $CHARM_DIR/files/archives/${relatedService//-}_mq_create_queue.mqsc"
			if [ $? == 0 ]; then
				juju-log "IBM MQ: Queue created."
			else
				juju-log "IBM MQ: Failed to create queue."
			fi

			QM_Name=$QMA
			Qname=$CHARM_DIR/files/archives/${relatedService//-}_mq_create_queue.mqsc
			port=$new_port
			open-port $port/TCP

			juju-log "create_QM_Queue values are QM_name=$QM_Name Qname=$Qname hostname=$hostname port=$port"
		fi
	elif [ $QM_exists -eq 1 ];then
		QM_Name=$QMA
		Qname=$CHARM_DIR/files/archives/${relatedService//-}_mq_create_queue.mqsc
		port=`ps -ef| grep runmqlsr | grep "$QMA" | grep -v grep | grep "su -l mqm" | awk -F"-p " '{print $2}'`
		juju-log "relation with ${relatedService} already exists"
		juju-log "create_QM_Queue values are QM_name=$QM_Name Qname=$Qname hostname=$hostname port=$port"
	fi

}


# IBM MQ: Deleting queue	
delete_QM_Queue()
{

	relatedService=$1

	QMA="${relatedService//-}.queue.manager"
	# Delete queue
	juju-log "IBM MQ: Deleting queue."
	su -l mqm -c "$MQ_INSTALL_PATH/bin/runmqsc $QMA < $CHARM_DIR/files/archives/${relatedService//-}_mq_delete_queue.mqsc"
	if [ $? == 0 ]; then
		juju-log "IBM MQ: Deleted queue."
	else
		juju-log "IBM MQ: Failed to delete queue."
	fi
	

	port=`ps -ef| grep runmqlsr | grep "$QMA" | grep -v grep | grep "su -l mqm" | awk -F"-p " '{print $2}'`

	juju-log "IBM MQ: stop Queue manager from listening on port"
	su -l mqm -c "$MQ_INSTALL_PATH/bin/endmqlsr -m $QMA -w"
	if [ $? == 0 ]; then
                juju-log "IBM MQ: Stopped queue manager from listening."
        else
                juju-log "IBM MQ: Failed to stop queue manager to listen."
        fi

	#close-port the open-port for the consumer
	close-port $port
	
	# End queue manger
	juju-log "IBM MQ: Stopping queue manager."
	su -l mqm -c "$MQ_INSTALL_PATH/bin/endmqm $QMA"
	su -l mqm -c "$MQ_INSTALL_PATH/bin/endmqlsr -w -m  $QMA"
	if [ $? -eq 0 ]; then
		juju-log "IBM MQ: Queue Manager is stopped."
	else
		juju-log "IBM MQ: Queue Manager failed to stop."
	fi

	sleep 10

	juju-log "IBM MQ: Deleting queue manager."
	su -l mqm -c "$MQ_INSTALL_PATH/bin/dltmqm $QMA"
	if [ $? -eq 0 ]; then
		juju-log "IBM MQ: Queue Manager is deleted."
	else
		juju-log "IBM MQ: Queue Manager could not be deleted."
	fi

	#clean up the created MQSC files
	if [ -f $CHARM_DIR/files/archives/${relatedService//-}_mq_create_queue.mqsc ];then
		sudo bash -c "rm $CHARM_DIR/files/archives/${relatedService//-}_mq_create_queue.mqsc"
	fi
	if [ -f $CHARM_DIR/files/archives/${relatedService//-}_mq_delete_queue.mqsc ]; then
		sudo bash -c "rm $CHARM_DIR/files/archives/${relatedService//-}_mq_delete_queue.mqsc"
	fi

	set -e
	juju-log "IBM MQ: Deleting queue completed."
}

@when 'ibm-base.license.accepted' 'ibm-base.curl.resource.fetched'
@when_not 'ibm-mq.installed'
function mq_install() {
	juju-log "inside mq install function."
	mq_inst=`is_mq_installed`
	juju-log "mq_inst==$mq_inst"
	if [ -f  $CHARM_DIR/files/archives/*.gz  ]; then
		juju-log "IBM MQ: Extracting IBM MQ package."
		tar xvfz $CHARM_DIR/files/archives/*.gz  --strip-components=1 -C $CHARM_DIR/files/archives 
		rm -rf  $CHARM_DIR/files/archives/*.gz
		juju-log "IBM MQ: Extracted IBM MQ package."
		juju-log "Awaiting acceptance of IBM MQ license (see README on how to accept the license)."
	fi
	if  [ $mq_inst == False ]; then
	# Check MQ package availability
		if [ -f  $CHARM_DIR/files/archives/MQSeriesServer*rpm ] && [ -f  $CHARM_DIR/files/archives/MQSeriesRuntime*rpm ]; 
		then 
			echo "MQ Packages available for installation."; 
			$CHARM_DIR/files/archives/mqlicense.sh -accept
			juju-log "Installing available MQ packages."
			rpm $RPM_INSTALL_ARG --prefix $MQ_INSTALL_PATH $CHARM_DIR/files/archives/MQSeries*rpm
			juju-log "Installation of available MQ packages complete."
			# Configure system values  for MQ
			configure_system
			set_state 'ibm-mq.installed'
			status-set active "MQ installed successfully"
		else 
			status-set blocked "MQ Packages missing. Please check README file Upgrade MQ charm after adding the MQ packages"
		fi
	fi
}

@when 'ibm-mq.installed'
@when 'messaging.connected'
function export_mq_details(){
	juju-log "on ibm_mq.connected triggered here"
	services=$(relation_call --state=messaging.connected services) || true
	for service in $services; do
		juju-log "$service"
		consumer_hostname=$(relation_call --state=messaging.connected get_consumer_hostname) || true
		create_QM_Queue $service $consumer_hostname
		juju-log "create_QM_Queue values are QM_name=$QM_Name Qname=$Qname hostname=$hostname port=$port"
		relation_call --state=messaging.connected set_mq_details $service $QM_Name $Qname $hostname $port || true
	done
}

@when 'messaging.departed' 'ibm-mq.installed'
function remove_mq_QM_Q() {
	juju-log "in delete_QM_Queue.."
	services=$(relation_call --state=messaging.departed services) || true
	for service in $services; do
		juju-log "$service"
		delete_QM_Queue $service
		juju-log "delete_QM_Queue done."
		juju-log "Calling dismiss for $service"
		relation_call --state=messaging.departed dismiss $service || true
	done
 }

@when 'ibm-mq.installed'
@when_not 'ibm-base.license.accepted'
function uninstall() {
	# Uninstall all versions of IBM MQ
	status-set maintenance "Uninstalling IBM MQ"
	mq_license_accepted=`config-get accept-ibm-mq-license`
	remove_software 
	remove_state 'ibm-mq.installed'
	remove_state 'messaging.connected'
	status-set blocked "IBM MQ Uninstalled"
}

reactive_handler_main
